@@ -6,12 +6,14 @@ use warnings;
 use Readonly;
 use Data::Dumper;
 use Log::Log4perl qw(get_logger :levels);
+use JSON::XS;
 
 BEGIN {
-    push(@INC, 'lib');
+    unshift(@INC, 'lib');
 };
 
 use Net::FreeIPA;
+use Net::FreeIPA::API;
 use Template;
 
 use Cwd qw(abs_path);
@@ -22,11 +24,11 @@ Readonly my $GEN_API_DIR => dirname(abs_path($0));
 
 Readonly my $SCRIPT_NAME => basename($0);
 
-Readonly my $MODULE_NAME => 'API';
+Readonly my $MODULE_NAME => 'Data';
 
 =head1 SYNOPSIS
 
-Generate the API.pm from JSON API
+Generate the API/Data.pm and Data.pod from JSON API
 
     GEN_API_DEBUG=1 GEN_API_HOSTNAME=host.example.com ./gen_api/gen_api.pl
 
@@ -56,13 +58,12 @@ sub get_api
     # most recent
     delete $f->{api_version};
 
-    $f->get_api_version() || die("Failed to get api_version ".Dumper($f));
-    my $version = $f->{result};
+    my $version = $f->get_api_version() || die("Failed to get api_version ".Dumper($f));
     $f->set_api_version($version);
 
-    $f->get_api_commands() || die("Failed to get commands metdata ".Dumper($f));
+    my $commands = $f->get_api_commands() || die("Failed to get commands metdata ".Dumper($f));
 
-    return $version, $f->{result};
+    return $version, $commands;
 }
 
 
@@ -94,15 +95,16 @@ my $f = Net::FreeIPA->new();
 
 sub make_module
 {
-    my ($text) = @_;
+    my ($text, $pod) = @_;
 
-    my $fn = "$GEN_API_DIR/../lib/Net/FreeIPA/$MODULE_NAME.pm";
+    my $fn = "$GEN_API_DIR/../lib/Net/FreeIPA/API/Data.";
+    $fn .= $pod ? 'pod' : 'pm';
 
-    open FH, "> $fn";
+    open FH, "> $fn" || die ("Failed to open $fn: $!");
     print FH $text;
     close FH;
 
-    print "Created module $fn\n"
+    print "Created $fn\n"
 }
 
 
@@ -116,23 +118,12 @@ sub main
     }) || die "$Template::ERROR\n";
 
     my $vars = {
-        prefix => $Net::FreeIPA::Convert::API_METHOD_PREFIX,
+        prefix => $Net::FreeIPA::API::API_METHOD_PREFIX,
         version => $version,
         commands => $commands,
         module_name => $MODULE_NAME,
         script_name => $SCRIPT_NAME,
-        join_name => sub {my $args = shift; return join(' ', map {$_->{name}} @$args); },
-        join_name_vars => sub {my $args = shift; return join(', ', map {'$'.$_->{name}} @$args)},
-        join_type => sub {
-            my $args = shift;
-            return join(' ',
-                        map {join(':',
-                                  $_->{type},
-                                  ($_->{multivalue} ? 1 : 0),
-                                  (($_->{required} && (! $_->{autofill})) ? 1 : 0), # only mandatory if required and no autofill/default
-                                 )} @$args);
-        },
-        ref => sub {return ref(shift) },
+        encode_json => sub { return encode_json(Net::FreeIPA::API::Magic::cache(shift));},
         check_hash => sub {
             my $array = shift;
             my @newarray;
@@ -140,25 +131,33 @@ sub main
                 if (ref($el) eq '') {
                     my $oldel = $el;
                     $oldel =~ s/[*+?]$//;
-                    $el = {
-                        name => $oldel,
-                        type => 'unknown',
-                        class => 'unknown',
-                        multivalue => 0,
-                        doc => 'unknown',
-                    };
-                }
+                    $el = { %Net::FreeIPA::API::Magic::CACHE_TAKES_DEFAULT };
+                    $el->{name} = $oldel;
+                    $el->{doc} = 'unknown';
+                };
                 push(@newarray, $el);
             };
             return \@newarray;
         }
     };
 
-    my $text = '';
-    $tt->process('api.tt', $vars, \$text)
-        || die $tt->error(), "\n";
+    # flush the cache, otherwise the encode_json TT command uses the cache data (leading to
+    # missing commands and interpreted data (so no identical JSON re-encoding))
+    Net::FreeIPA::API::Magic::flush_cache();
+
+    # pod first, the data.tt uses encode_json which modifies the commands hashref
+    my $pod = '';
+    $tt->process('pod.tt', $vars, \$pod)
+        || die "POD error ", $tt->error(), "\n";
+
+    make_module($pod, 1);
+
+    my $api = '';
+    $tt->process('data.tt', $vars, \$api)
+        || die "API error ", $tt->error(), "\n";
+
+    make_module($api);
 
-    make_module($text);
 }
 
 
